"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pipeline = void 0;
const compose_1 = require("@zodash/compose");
const emitter_1 = require("@zodash/emitter");
class Pipeline extends emitter_1.Emitter {
    constructor(config) {
        super();
        this.config = config;
        this.ctx = this.createContext();
        this.use(async (ctx, next) => {
            const currentStep = this.getCurrentStep();
            const progress = this.getCurrentProgress();
            try {
                this.emit('stepStart', currentStep, progress);
                this.updateCurrentStepStatus('running');
                await next();
                this.emit('stepFinish', currentStep, progress);
                this.updateCurrentStepStatus('success');
            }
            catch (error) {
                this.emit('error', error, currentStep, progress);
                this.updateCurrentStepStatus('error');
                // pipeline should break
                throw error;
            }
        });
    }
    createContext() {
        return {
            name: this.config.name,
            config: this.config,
            middlewares: [],
            steps: null,
            progress: null,
        };
    }
    createHandler() {
        return compose_1.compose(...this.ctx.middlewares);
    }
    setSteps(steps) {
        const { ctx } = this;
        ctx.steps = steps.map((step, index) => ({
            ...step,
            index,
            status: 'pending',
        }));
        this.resetProgress();
    }
    resetProgress() {
        const { ctx } = this;
        ctx.progress = {
            total: ctx.steps.length,
            current: 0,
            percent: 0,
        };
    }
    setCurrentStep(step) {
        const { ctx } = this;
        ctx.progress.current = step.index;
    }
    setCurrentProgress() {
        const { ctx } = this;
        ctx.progress.percent = ~~(((ctx.progress.current + 1) / ctx.progress.total) *
            100);
    }
    getCurrentStep() {
        const { ctx } = this;
        return ctx.steps[ctx.progress.current];
    }
    getCurrentProgress() {
        const { ctx } = this;
        return ctx.progress;
    }
    updateCurrentStepStatus(status) {
        const step = this.getCurrentStep();
        step.status = status;
    }
    get name() {
        return this.config.name;
    }
    use(...middleware) {
        this.ctx.middlewares = this.ctx.middlewares.concat(middleware);
        return this;
    }
    addProcessor(processor) {
        this.use(async (ctx, next) => {
            const currentStep = this.getCurrentStep();
            const progress = this.getCurrentProgress();
            await processor.apply(null, [currentStep, progress, ctx]);
            await next();
        });
    }
    async run(steps) {
        const { ctx } = this;
        try {
            this.emit('start', ctx);
            this.emit('prepareStart', ctx);
            const handler = this.createHandler();
            this.emit('prepareFinish', ctx);
            this.setSteps(steps);
            this.emit('progress', this.getCurrentProgress(), this.getCurrentStep());
            for (const step of ctx.steps) {
                this.setCurrentStep(step);
                // this.emit('progress', this.getCurrentProgress(), this.getCurrentStep());
                await handler.apply(this, [ctx]);
                this.setCurrentProgress();
                this.emit('progress', this.getCurrentProgress(), this.getCurrentStep());
            }
            this.emit('finish', ctx);
        }
        catch (error) {
            this.emit('fatal', error);
        }
    }
}
exports.Pipeline = Pipeline;
//# sourceMappingURL=core.js.map